package com.ningdatech.pmapi.todocenter.utils;

import com.itextpdf.text.Image;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.*;
import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.util.*;

/**
 * pdf 生成工具
 *
 * @author CMM
 * @since 2023/03/13 13:41
 */
@Slf4j
public class PdfGenerateUtil {

    private PdfGenerateUtil() {
    }


    /**
     * 生成填充了模板参数的pdf
     *
     * @param templatePdfInputStream 模板pdf流
     * @param paramsMap            填充参数
     * @return
     */
    public static byte[] generatePdf(InputStream templatePdfInputStream,Map<String, Object> paramsMap) throws IOException {
        byte[] templatePdfByteArray = readBytes(templatePdfInputStream);
        return generatePdf(templatePdfByteArray, paramsMap);
    }

    /**
     * 生成填充了模板参数的pdf
     *
     * @param templatePdfByteArray 模板pdf字节数组
     * @param paramsMap            填充参数
     * @return
     */
    public static byte[] generatePdf(byte[] templatePdfByteArray, Map<String, Object> paramsMap) {

        PdfReader reader = null;
        ByteArrayOutputStream bos = null;
        try {
            //创建书写器，用于往document中书写信息
            // 通过本地文件路径获取资源
            reader = new PdfReader(templatePdfByteArray);
            bos = new ByteArrayOutputStream();
            PdfStamper stamper = new PdfStamper(reader, bos);

            //使用中文字体
            BaseFont baseFont = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
            ArrayList<BaseFont> fontList = new ArrayList<>();
            fontList.add(baseFont);


            AcroFields form = stamper.getAcroFields();
            form.setSubstitutionFonts(fontList);

            for (String fieldName : paramsMap.keySet()) {
                if (paramsMap.get(fieldName) == null) {
                    continue;
                }

                if (fieldName.indexOf("Image") > 0) {
                    String imgPath = paramsMap.get(fieldName).toString();
                    int pageNo = form.getFieldPositions(fieldName).get(0).page;
                    Rectangle rectangle = form.getFieldPositions(fieldName).get(0).position;
                    float x = rectangle.getLeft();
                    float y = rectangle.getTop();
                    //根据路径读取图片
                    Image image = Image.getInstance(imgPath);
                    //获取图片页面
                    PdfContentByte under = stamper.getOverContent(pageNo);
                    //图片大小自适应
                    image.scaleToFit(rectangle.getWidth(), rectangle.getHeight());
                    //添加图片
                    image.setAbsolutePosition(x, y - rectangle.getHeight());
                    under.addImage(image);
                } else {
                    // 设置占位字段
                    form.setField(fieldName, paramsMap.get(fieldName).toString());
                }
            }

            stamper.setFormFlattening(false);
            stamper.close();

        } catch (Exception e) {
            log.error("通过模板生成PDF失败", e.getMessage());
            return null;
        } finally {
            try {
                if (null != reader) {
                    reader.close();
                }
                if (null != bos) {
                    bos.close();
                }

            } catch (IOException e) {
                log.error("close resource error", e.getMessage());
            }
        }

        return bos.toByteArray();
    }

    /**
     * 读取输入流到字节数组
     *
     * @param in
     * @return
     * @throws IOException
     */
    private static byte[] readBytes(InputStream in) throws IOException {
        //读取字节的缓冲
        byte[] buffer = new byte[1024];
        //最终的数据
        byte[] result = new byte[0];
        int size = 0;
        while ((size = in.read(buffer)) != -1) {
            int oldLen = result.length;
            byte[] tmp = new byte[oldLen + size];
            if (oldLen > 0) {//copy 旧字节
                System.arraycopy(result, 0, tmp, 0, oldLen);
            }
            //copy 新字节
            System.arraycopy(buffer, 0, tmp, oldLen, size);

            result = tmp;
        }
        return result;
    }
}


